"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _reactNative = require("react-native");

var _Ionicons = _interopRequireDefault(require("react-native-vector-icons/Ionicons"));

var _styles = require("./styles");

var _lodash = _interopRequireDefault(require("lodash"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/* eslint-disable no-shadow */
const defaultProps = {
  style: {},
  textStyle: {},
  iconColor: 'black'
};

const CheckboxTreeComponent = /*#__PURE__*/_react.default.forwardRef((props, ref) => {
  const {
    keyField,
    textField,
    childField,
    childCheckField,
    style,
    textStyle,
    iconColor = 'black',
    iconSize = 26,
    openIcon,
    closeIcon,
    checkIcon,
    unCheckIcon,
    autoSelectParents = true,
    autoSelectChilds = true,
    renderItem
  } = props;
  const [listData, setListData] = (0, _react.useState)([]);
  const [selectedData, setSelectedData] = (0, _react.useState)([]);
  const [showsData, setShowsData] = (0, _react.useState)([]);
  (0, _react.useImperativeHandle)(ref, () => {
    return {
      clear
    };
  });
  (0, _react.useEffect)(() => {
    updateListData(props.data || []);
  }, [props.data]);
  (0, _react.useEffect)(() => {
    updateSelectedData(props.selectedData || []);
  }, [props.selectedData]);
  (0, _react.useEffect)(() => {
    updateShowsData(props.showsData || []);
  }, [props.showsData]);
  (0, _react.useEffect)(() => {
    if (typeof props.onSelect == 'function') {
      props.onSelect(selectedData);
    }
  }, [selectedData]);
  (0, _react.useEffect)(() => {
    if (typeof props.onShow == 'function') {
      props.onShow(showsData);
    }
  }, [showsData]);

  const clear = () => {
    setSelectedData([]);
    setShowsData([]);
  };

  const _renderIcon = status => {
    if (status) {
      if (checkIcon) {
        return checkIcon;
      } else {
        return /*#__PURE__*/_react.default.createElement(_Ionicons.default, {
          name: "ios-checkbox-outline",
          size: iconSize,
          color: iconColor
        });
      }
    } else {
      if (unCheckIcon) {
        return unCheckIcon;
      } else {
        return /*#__PURE__*/_react.default.createElement(_Ionicons.default, {
          name: "stop-outline",
          size: iconSize,
          color: iconColor
        });
      }
    }
  };

  const _exits = (list, value) => {
    const idx = list.indexOf(value[keyField]);
    return idx >= 0;
  };

  const setParents = tmpData => {
    var _tmpData$childField;

    (_tmpData$childField = tmpData[childField]) === null || _tmpData$childField === void 0 ? void 0 : _tmpData$childField.map(item => {
      item.parent = tmpData;
      item = setParents(item);
      return item;
    });
    return tmpData;
  };

  const updateListData = newData => {
    newData = newData.map(item => setParents(item));
    setListData([...newData]);
  };

  const updateSelectedData = newData => {
    let tmpData = _lodash.default.uniq(newData);

    setSelectedData([...tmpData]);
  };

  const updateShowsData = newData => {
    let tmpData = _lodash.default.uniq(newData);

    setShowsData([...tmpData]);
  };

  const getIds = list => {
    var tmp = [];
    if (!list) return tmp;

    if (Array.isArray(list)) {
      list.map(item => {
        var _item$childField;

        tmp.push(item[keyField]);
        const existsChild = childCheckField ? item[childCheckField] : ((_item$childField = item[childField]) === null || _item$childField === void 0 ? void 0 : _item$childField.length) > 0;

        if (existsChild) {
          const childs = getIds(item[childField]);
          tmp = [...tmp, ...childs];
        }

        ;
      });
    }

    return tmp;
  };

  const removeItem = value => {
    const idx = selectedData.indexOf(value);
    if (idx >= 0) selectedData.splice(idx, 1);
  };

  const parent = (item, isTick) => {
    if (!item) return;

    if (item[childField]) {
      if (isTick) {
        const check = item[childField].filter(child => !_exits(selectedData, child));
        if (check.length === 0) selectedData.push(item[keyField]);
      } else {
        removeItem(item[keyField]);
      }
    }

    parent(item.parent, isTick);
  };

  const onTick = item => {
    var childs = [];
    selectedData.push(item[keyField]);
    if (autoSelectParents) parent(item.parent, true);
    if (autoSelectChilds) childs = getIds(item[childField]);
    var tmp = [...selectedData, ...childs];
    updateSelectedData(tmp);
  };

  const onUnTick = item => {
    removeItem(item[keyField]);
    if (autoSelectParents) parent(item.parent, false);

    if (autoSelectChilds) {
      const childs = getIds(item[childField]);
      childs.map(item => removeItem(item));
    }

    updateSelectedData(selectedData);
  };

  const showChild = item => {
    const openIndex = showsData.findIndex(dItem => dItem == item[keyField]);
    var tmp = showsData;
    if (openIndex >= 0) tmp.splice(openIndex, 1);else tmp.push(item[keyField]);
    updateShowsData(tmp);
  };

  const renderList = (item, childs, index) => {
    const isTick = selectedData.findIndex(dItem => dItem == item[keyField]) >= 0;
    const isOpen = showsData.findIndex(dItem => dItem == item[keyField]) >= 0;
    return /*#__PURE__*/_react.default.createElement(_reactNative.View, {
      style: [_styles.styles.item, {
        marginLeft: iconSize
      }],
      key: index
    }, renderItem ? renderItem({
      item: item,
      isSelect: isTick,
      isOpen: isOpen,
      onOpen: () => showChild(item),
      onClose: () => showChild(item),
      onSelect: () => {
        isTick ? onUnTick(item) : onTick(item);
      }
    }) : /*#__PURE__*/_react.default.createElement(_reactNative.View, {
      style: _styles.styles.rowItem
    }, childs && childs.length > 0 ? /*#__PURE__*/_react.default.createElement(_reactNative.TouchableOpacity, {
      onPress: () => showChild(item)
    }, isOpen ? openIcon ? openIcon : /*#__PURE__*/_react.default.createElement(_Ionicons.default, {
      name: "ios-remove",
      size: iconSize,
      color: iconColor
    }) : closeIcon ? closeIcon : /*#__PURE__*/_react.default.createElement(_Ionicons.default, {
      name: "add-outline",
      size: iconSize,
      color: iconColor
    })) : /*#__PURE__*/_react.default.createElement(_reactNative.Text, {
      style: {
        width: iconSize
      }
    }, `  `), /*#__PURE__*/_react.default.createElement(_reactNative.TouchableOpacity, {
      style: _styles.styles.flex1,
      onPress: () => {
        if (isTick) onUnTick(item);else onTick(item);
      }
    }, /*#__PURE__*/_react.default.createElement(_reactNative.View, {
      style: _styles.styles.center
    }, _renderIcon(isTick), /*#__PURE__*/_react.default.createElement(_reactNative.Text, {
      style: [_styles.styles.name, textStyle],
      numberOfLines: 3
    }, item[textField])))), isOpen && /*#__PURE__*/_react.default.createElement(_reactNative.View, {
      style: [!isOpen && _styles.styles.height0]
    }, childs && childs.map((data, index) => renderList(data, data[childField], index))));
  };

  return /*#__PURE__*/_react.default.createElement(_reactNative.View, {
    style: [_styles.styles.container, style]
  }, /*#__PURE__*/_react.default.createElement(_reactNative.FlatList, {
    data: listData,
    renderItem: _ref => {
      let {
        item,
        index
      } = _ref;
      return renderList(item, item[childField], index);
    },
    keyExtractor: (_item, index) => index.toString(),
    showsVerticalScrollIndicator: false
  }));
});

CheckboxTreeComponent.defaultProps = defaultProps;
var _default = CheckboxTreeComponent;
exports.default = _default;
//# sourceMappingURL=index.js.map