/* eslint-disable no-shadow */
import React, { useEffect, useImperativeHandle, useState } from 'react';
import { FlatList, TouchableOpacity, View, Text } from 'react-native';
import Ionicons from 'react-native-vector-icons/Ionicons';
import { styles } from './styles';
import _ from 'lodash';
const defaultProps = {
  style: {},
  textStyle: {},
  iconColor: 'black'
};
const CheckboxTreeComponent = /*#__PURE__*/React.forwardRef((props, ref) => {
  const {
    keyField,
    textField,
    childField,
    childCheckField,
    style,
    textStyle,
    iconColor = 'black',
    iconSize = 26,
    openIcon,
    closeIcon,
    checkIcon,
    unCheckIcon,
    autoSelectParents = true,
    autoSelectChilds = true,
    renderItem
  } = props;
  const [listData, setListData] = useState([]);
  const [selectedData, setSelectedData] = useState([]);
  const [showsData, setShowsData] = useState([]);
  useImperativeHandle(ref, () => {
    return {
      clear
    };
  });
  useEffect(() => {
    updateListData(props.data || []);
  }, [props.data]);
  useEffect(() => {
    updateSelectedData(props.selectedData || []);
  }, [props.selectedData]);
  useEffect(() => {
    updateShowsData(props.showsData || []);
  }, [props.showsData]);
  useEffect(() => {
    if (typeof props.onSelect == 'function') {
      props.onSelect(selectedData);
    }
  }, [selectedData]);
  useEffect(() => {
    if (typeof props.onShow == 'function') {
      props.onShow(showsData);
    }
  }, [showsData]);

  const clear = () => {
    setSelectedData([]);
    setShowsData([]);
  };

  const _renderIcon = status => {
    if (status) {
      if (checkIcon) {
        return checkIcon;
      } else {
        return /*#__PURE__*/React.createElement(Ionicons, {
          name: "ios-checkbox-outline",
          size: iconSize,
          color: iconColor
        });
      }
    } else {
      if (unCheckIcon) {
        return unCheckIcon;
      } else {
        return /*#__PURE__*/React.createElement(Ionicons, {
          name: "stop-outline",
          size: iconSize,
          color: iconColor
        });
      }
    }
  };

  const _exits = (list, value) => {
    const idx = list.indexOf(value[keyField]);
    return idx >= 0;
  };

  const setParents = tmpData => {
    var _tmpData$childField;

    (_tmpData$childField = tmpData[childField]) === null || _tmpData$childField === void 0 ? void 0 : _tmpData$childField.map(item => {
      item.parent = tmpData;
      item = setParents(item);
      return item;
    });
    return tmpData;
  };

  const updateListData = newData => {
    newData = newData.map(item => setParents(item));
    setListData([...newData]);
  };

  const updateSelectedData = newData => {
    let tmpData = _.uniq(newData);

    setSelectedData([...tmpData]);
  };

  const updateShowsData = newData => {
    let tmpData = _.uniq(newData);

    setShowsData([...tmpData]);
  };

  const getIds = list => {
    var tmp = [];
    if (!list) return tmp;

    if (Array.isArray(list)) {
      list.map(item => {
        var _item$childField;

        tmp.push(item[keyField]);
        const existsChild = childCheckField ? item[childCheckField] : ((_item$childField = item[childField]) === null || _item$childField === void 0 ? void 0 : _item$childField.length) > 0;

        if (existsChild) {
          const childs = getIds(item[childField]);
          tmp = [...tmp, ...childs];
        }

        ;
      });
    }

    return tmp;
  };

  const removeItem = value => {
    const idx = selectedData.indexOf(value);
    if (idx >= 0) selectedData.splice(idx, 1);
  };

  const parent = (item, isTick) => {
    if (!item) return;

    if (item[childField]) {
      if (isTick) {
        const check = item[childField].filter(child => !_exits(selectedData, child));
        if (check.length === 0) selectedData.push(item[keyField]);
      } else {
        removeItem(item[keyField]);
      }
    }

    parent(item.parent, isTick);
  };

  const onTick = item => {
    var childs = [];
    selectedData.push(item[keyField]);
    if (autoSelectParents) parent(item.parent, true);
    if (autoSelectChilds) childs = getIds(item[childField]);
    var tmp = [...selectedData, ...childs];
    updateSelectedData(tmp);
  };

  const onUnTick = item => {
    removeItem(item[keyField]);
    if (autoSelectParents) parent(item.parent, false);

    if (autoSelectChilds) {
      const childs = getIds(item[childField]);
      childs.map(item => removeItem(item));
    }

    updateSelectedData(selectedData);
  };

  const showChild = item => {
    const openIndex = showsData.findIndex(dItem => dItem == item[keyField]);
    var tmp = showsData;
    if (openIndex >= 0) tmp.splice(openIndex, 1);else tmp.push(item[keyField]);
    updateShowsData(tmp);
  };

  const renderList = (item, childs, index) => {
    const isTick = selectedData.findIndex(dItem => dItem == item[keyField]) >= 0;
    const isOpen = showsData.findIndex(dItem => dItem == item[keyField]) >= 0;
    return /*#__PURE__*/React.createElement(View, {
      style: [styles.item, {
        marginLeft: iconSize
      }],
      key: index
    }, renderItem ? renderItem({
      item: item,
      isSelect: isTick,
      isOpen: isOpen,
      onOpen: () => showChild(item),
      onClose: () => showChild(item),
      onSelect: () => {
        isTick ? onUnTick(item) : onTick(item);
      }
    }) : /*#__PURE__*/React.createElement(View, {
      style: styles.rowItem
    }, childs && childs.length > 0 ? /*#__PURE__*/React.createElement(TouchableOpacity, {
      onPress: () => showChild(item)
    }, isOpen ? openIcon ? openIcon : /*#__PURE__*/React.createElement(Ionicons, {
      name: "ios-remove",
      size: iconSize,
      color: iconColor
    }) : closeIcon ? closeIcon : /*#__PURE__*/React.createElement(Ionicons, {
      name: "add-outline",
      size: iconSize,
      color: iconColor
    })) : /*#__PURE__*/React.createElement(Text, {
      style: {
        width: iconSize
      }
    }, `  `), /*#__PURE__*/React.createElement(TouchableOpacity, {
      style: styles.flex1,
      onPress: () => {
        if (isTick) onUnTick(item);else onTick(item);
      }
    }, /*#__PURE__*/React.createElement(View, {
      style: styles.center
    }, _renderIcon(isTick), /*#__PURE__*/React.createElement(Text, {
      style: [styles.name, textStyle],
      numberOfLines: 3
    }, item[textField])))), isOpen && /*#__PURE__*/React.createElement(View, {
      style: [!isOpen && styles.height0]
    }, childs && childs.map((data, index) => renderList(data, data[childField], index))));
  };

  return /*#__PURE__*/React.createElement(View, {
    style: [styles.container, style]
  }, /*#__PURE__*/React.createElement(FlatList, {
    data: listData,
    renderItem: _ref => {
      let {
        item,
        index
      } = _ref;
      return renderList(item, item[childField], index);
    },
    keyExtractor: (_item, index) => index.toString(),
    showsVerticalScrollIndicator: false
  }));
});
CheckboxTreeComponent.defaultProps = defaultProps;
export default CheckboxTreeComponent;
//# sourceMappingURL=index.js.map